package fr.llibre.ml2;
// Extrait de la classe R3 du Cinespace/mllpack

public class R3 {
	
	/** Génération d'un vecteur de dim 3
    @param V     V[3]   : vecteur résultat
    @param x
    @param y
    @param z
    @return    référence sur le résultat
	 */
	public static double []set3(double []v, double x, double y, double z)
	{
		v[0] = x;
		v[1] = y;
		v[2] = z;
		return v;
	}
	
	
	/** Génération d'une matrice 3*3 double tableau
	 @param r     r[3][3]   :  tableau résultat
	 @param r00   1ere ligne, 1ere composante
	 @param r01   1ere ligne, 2eme composante
	 @param r02   1ere ligne, 3eme composante
	 @param r10   2eme ligne, 1ere composante ....
	 @return r    reference sur le resultat
	 */
	public static double [][] set33(double [][]r, double r00, double r01, double r02,
			double r10, double r11, double r12,
			double r20, double r21, double r22)
	{
		r[0][0] = r00; r[0][1] = r01; r[0][2] = r02;
		r[1][0] = r10; r[1][1] = r11; r[1][2] = r12;
		r[2][0] = r20; r[2][1] = r21; r[2][2] = r22;
		return r;
	}

	/** Multiplication mat33 x v3 : r = a*b
	 @param r     r[3]    : vecteur résultat
	 @param a     a[3][3] : matrice multiplicatrice
	 @param b     b[3]    : vecteur multipliée
	 */
	public static void madvec(double []r, double [][]a, double []b)
	{
		double x, y;
		x    = a[0][0]*b[0] + a[0][1]*b[1] + a[0][2]*b[2];
		y    = a[1][0]*b[0] + a[1][1]*b[1] + a[1][2]*b[2];
		r[2] = a[2][0]*b[0] + a[2][1]*b[1] + a[2][2]*b[2];
	
		r[0] = x;
		r[1] = y;
	}
	
	/** Multiplication mat33Transp x v3 : r = a'*b
	 @param r     r[3]    : vecteur résultat
	 @param a     a[3][3] : matrice multiplicatrice
	 @param b     b[3]    : vecteur multipliée
	 */
	public static void matvec(double []r, double [][]a, double []b)
	{
		double x, y;
		x    = a[0][0]*b[0] + a[1][0]*b[1] + a[2][0]*b[2];
		y    = a[0][1]*b[0] + a[1][1]*b[1] + a[2][1]*b[2];
		r[2] = a[0][2]*b[0] + a[1][2]*b[1] + a[2][2]*b[2];
	
		r[0] = x;
		r[1] = y;
	}	
	

	/** Multiplication mat33 x mat33 : r = a*b
	 @param r     r[3][3] : matrice résultat
	 @param a     a[3][3] : matrice multiplicatrice
	 @param b     b[3][3] : matrice multipliée
	 */
	public static void madmad(double [][]r, double [][]a, double [][]b)
	{
		double r00, r01, r02, r10, r11, r12, r20, r21, r22;
		r00 = a[0][0]*b[0][0] + a[0][1]*b[1][0] + a[0][2]*b[2][0];
		r01 = a[0][0]*b[0][1] + a[0][1]*b[1][1] + a[0][2]*b[2][1];
		r02 = a[0][0]*b[0][2] + a[0][1]*b[1][2] + a[0][2]*b[2][2];

		r10 = a[1][0]*b[0][0] + a[1][1]*b[1][0] + a[1][2]*b[2][0];
		r11 = a[1][0]*b[0][1] + a[1][1]*b[1][1] + a[1][2]*b[2][1];
		r12 = a[1][0]*b[0][2] + a[1][1]*b[1][2] + a[1][2]*b[2][2];

		r20 = a[2][0]*b[0][0] + a[2][1]*b[1][0] + a[2][2]*b[2][0];
		r21 = a[2][0]*b[0][1] + a[2][1]*b[1][1] + a[2][2]*b[2][1];
		r22 = a[2][0]*b[0][2] + a[2][1]*b[1][2] + a[2][2]*b[2][2];

		r[0][0] = r00; r[0][1] = r01; r[0][2] = r02;
		r[1][0] = r10; r[1][1] = r11; r[1][2] = r12;
		r[2][0] = r20; r[2][1] = r21; r[2][2] = r22;
	}

	/** Multiplication mat33T x mat33 : r = a'*b
	 @param r     r[3][3] : matrice résultat
	 @param a     a[3][3] : matrice multiplicatrice
	 @param b     b[3][3] : matrice multipliée
	 */
	public static void matmad(double [][]r, double [][]a, double [][]b)
	{
		double r00, r01, r02, r10, r11, r12, r20, r21, r22;
		r00 = a[0][0]*b[0][0] + a[1][0]*b[1][0] + a[2][0]*b[2][0];
		r01 = a[0][0]*b[0][1] + a[1][0]*b[1][1] + a[2][0]*b[2][1];
		r02 = a[0][0]*b[0][2] + a[1][0]*b[1][2] + a[2][0]*b[2][2];

		r10 = a[0][1]*b[0][0] + a[1][1]*b[1][0] + a[2][1]*b[2][0];
		r11 = a[0][1]*b[0][1] + a[1][1]*b[1][1] + a[2][1]*b[2][1];
		r12 = a[0][1]*b[0][2] + a[1][1]*b[1][2] + a[2][1]*b[2][2];

		r20 = a[0][2]*b[0][0] + a[1][2]*b[1][0] + a[2][2]*b[2][0];
		r21 = a[0][2]*b[0][1] + a[1][2]*b[1][1] + a[2][2]*b[2][1];
		r22 = a[0][2]*b[0][2] + a[1][2]*b[1][2] + a[2][2]*b[2][2];

		r[0][0] = r00; r[0][1] = r01; r[0][2] = r02;
		r[1][0] = r10; r[1][1] = r11; r[1][2] = r12;
		r[2][0] = r20; r[2][1] = r21; r[2][2] = r22;
	}

	/*---------------------------------------------------------------------
            Matrice d'attitude d'un repère de visée
       ------------------------------------------------------------

    Entrees:
        double azi	: Azimut de la ligne de visée
        double sit  : Site de la ligne de visée

    Sorties:
        mat : la matrice de passage du repère fixe au repère de visée

        Direction de visée selon -Z (l'axe +Z est dirigé vers l'oeil).
        l'axe X est horizontal et Y est dans le plan vertical contenant Z
        Y dirigé vers le haut, si |sit] < PI/2
    ----------------------------------------------------------------------*/
	public static void asiTom3(double [][]mat, double azi, double sit)
	{
		double ca, sa, cs, ss;
		ca = Math.cos(azi); sa = Math.sin(azi);
		cs = Math.cos(sit); ss = Math.sin(sit);

		mat[0][0] = sa;    mat[0][1] =-ca*ss; mat[0][2] =-ca*cs;
		mat[1][0] = -ca;   mat[1][1] =-sa*ss; mat[1][2] =-sa*cs;
		mat[2][0] = 0.;    mat[2][1] = cs;    mat[2][2] =-ss;
	}

	/*---------------------------------------------------------------------
            Matrice d'attitue d'un repère local sur Terre
       ------------------------------------------------------------

    Entrees:
        double lon	: longitude / greenwich
        double lat  : latitude géodésique

    Sorties:
        mat : la matrice de passage du repère de Greenwich au repère local
        [GA] telle que V_g = [GA] V_a

        Pour le repère local Ra, Z est vers le zénit, X vers le Sud et
        Y est vers l'est
    ----------------------------------------------------------------------*/
	public static void lolaAstroTom3(double [][]mat, double lon, double lat)
	{
		double cl = Math.cos(lon);
		double sl = Math.sin(lon);
		double sf = Math.sin(lat);
		double cf = Math.cos(lat);
		mat[0][0] =  cl*sf; mat[0][1] = -sl; mat[0][2] = cl*cf;
		mat[1][0] =  sl*sf; mat[1][1] =  cl; mat[1][2]= sl*cf;
		mat[2][0] =    -cf; mat[2][1] =  0.; mat[2][2] =    sf;
	}

}
