package fr.llibre.ml2;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;


/**
 * La classe Sater est une classe qui permet de definir la position d'un satellite terrestre
 *
 * @author Michel Llibre - ONERA/DCSD.
 */

public class Sater {

	private static final double Terre_GM = 3.986004418e14; // Cte. gravitation de la Terre
//	private static final double Terre_rT = 6378136.6;
	private static final double Terre_J2 = 0.0010826359; //Harmonique spherique J2 du potentiel Terrestre	
	private static final double Terre_r2J2 = Terrien.Terre_rT*Terrien.Terre_rT*Terre_J2;
	private static final double toRadians = Math.PI/180.;
	private static final double coef1 = 2.* Math.PI/86400.;
	private static final double coef2 = coef1/86400.;
	private static final double coef3 = coef2/86400.;

	// paramètres du satellite
	public String name;  // Nom du satellite
	private double ju0;  // jour julien des elements orbitaux
	private double e0;    // Excentricite
	private double i0;    // Inclinaison (radians)
	private double om0;   // Argument du perigee en radians
	private double gdOm0; // Ascension droite du noeud ascendant en radians à t0
	private double grand_Omega; // Ascension droite du noeud ascendant en radians
	//    private double rume2;
	private double ci0, si0;
	private double [][] m33_HN = new double [3][3];
	private double [][] m33_GL = new double [3][3];
	private double a0;     // Demi grand axe (m)
	//    private double delta1; // Coef de correction du a J2
	private double gdOmP;  // d/dt(Ascension_droite) en rd/s
	private double omP;    // d/dt(arg_perigee) en rd/s
	//    private double nnod;   // Mouvement nodal (n0 + omP) en rd/s
	//    private double Tn0;    // Periode mouvement moyen en rd/s
	//    private double Tnod;   // Periode nodale en secondes
	private double n0;     // Mouvement moyen en rd/s
	private double M0;	   // Anomalie moyenne à ju0
	private double n0dots2;   // 1/2 d/dt (nO)
	private double n0ddots6;// 1/6 d2/dt2(n0)    
	// Coordonnées equatoriales calculées après ju2equa
	public double distTS;
	public double longiSat_rd, latiSat_rd;  
	public double []v3_TS_g = new double [3];
	// Coordonnées topographiques relatives à la postobsv
//	private Postobsv postobsv;
	double []v3_AS_a = new double [3]; 
	double azi_dg, haut_dg; // calculées par makeTopo()


	// Creation d'un satellite à partir de ses TLE 	
	public Sater (TwoLE tle)
	{
		name = tle.name;
		n0 = tle.meanmo*coef1;        
		e0 = tle.eccn;              // Excentricite
		i0 = toRadians*tle.incl;    // Inclinaison (radians)
		om0 = toRadians*tle.argper; // Argument du perigee en radians
		gdOm0 = toRadians*tle.raan;	// Ascension droite du noeud ascendant en radians
		double a1 = Math.pow(Terre_GM/(n0*n0),(1./3.));
		double ume2 = 1.- e0*e0;
		//    	rume2 = Math.sqrt(ume2);
		ci0 = Math.cos(i0);
		si0 = Math.sin(i0);
		R3.set33(m33_HN,1, 0, 0, 0, ci0, -si0, 0, si0, ci0);
		a0 = a1;                    // Demi grand axe (m)
		double p = a0*ume2;
		double rT2J2sp2 = Terre_r2J2/(p*p);
		//        delta1 =  0.75*rT2J2sp2*rume2*(3.*ci0*ci0-1.); // Coef de correction du a J2
		double temp = rT2J2sp2*n0;
		gdOmP = -1.5*temp*ci0;		// d/dt(Ascension_droite) en rd/s
		omP = 0.75*temp*(5.*ci0*ci0-1.); // d/dt(arg_perigee) en rd/s
		//        nnod = n0 + omP;        // Mouvement nodal (n0 + omP) en rd/s
		//        Tn0 = 2*Math.PI/n0;     // Periode mouvement moyen en rd/s
		//        Tnod = 2*Math.PI/nnod;  // Periode nodale en secondes                   
		M0 = toRadians*tle.meanan;// Mouvement moyen en rd/s
		n0dots2 = tle.drag*coef2;   // 1/2 d/dt (nO)
		n0ddots6 = tle.nddot6*coef3;// 1/6 d2/dt2(n0)
		// jour julien des elements orbitaux	
		ju0 = Utils.dat2jul(tle.year, 1, 1.) + tle.refepoch - 1.;  	
	}

	private void ju2distTS(double ju)
	{  
		double t = (ju - ju0)*86400; // Nb de secondes depuis ju0
		grand_Omega = gdOm0 + gdOmP*t;
		//Mouvement moyen
		double n = n0 + t*(2.*n0dots2 + t*3.*n0ddots6);
		double a = a0 * Math.pow(n0/n, 2./3.);
		double e = 1. - a0*(1. - e0)/a;
		if (e < 0.) e = 0.;
		double unme2 = 1.- e*e;
		double p = a*unme2;
		double anoMoy_M = M0 + t*(n0 + t*(n0dots2 + t*n0ddots6));
		// Resolution equation de Kepler par iterations
		double anoExc_E = anoMoy_M;
		for (int iter = 0; iter < 4; iter++)
		{ 
			double dM = anoMoy_M - (anoExc_E - e* Math.sin(anoExc_E));
			if (Math.abs(dM) < 1.e-12) break;
			double dE = dM / (1. - e* Math.cos(anoExc_E) ) ;
			anoExc_E += dE;
			if (Math.abs(dE) < 1.e-12) break;
		}
		double cE = Math.cos(anoExc_E);
		double umecE = 1. - e*cE;
		double cv = (cE - e)/umecE;
		double unpecosv = 1. + e*cv;
		distTS = p/unpecosv;	
	}

	public void update(double ju)
	{
		double t = (ju - ju0)*86400; // Nb de secondes depuis ju0
		grand_Omega = gdOm0 + gdOmP*t;
		double petit_omega = om0 + omP*t;
		//Mouvement moyen
		double n = n0 + t*(2.*n0dots2 + t*3.*n0ddots6);
		double a = a0 * Math.pow(n0/n, 2./3.);
		double e = 1. - a0*(1. - e0)/a;
		if (e < 0.) e = 0.;
		double unme2 = 1.- e*e;
		double p = a*unme2;
		double sq_unme2 = Math.sqrt(unme2);
		double anoMoy_M = M0 + t*(n0 + t*(n0dots2 + t*n0ddots6));
		// Resolution equation de Kepler par iterations
		double anoExc_E = anoMoy_M;
		for (int iter = 0; iter < 4; iter++)
		{ 
			double dM = anoMoy_M - (anoExc_E - e* Math.sin(anoExc_E));
			if (Math.abs(dM) < 1.e-12) break;
			double dE = dM / (1. - e* Math.cos(anoExc_E) ) ;
			anoExc_E += dE;
			if (Math.abs(dE) < 1.e-12) break;
		}
		double cE = Math.cos(anoExc_E);
		double sE = Math.sin(anoExc_E);
		double umecE = 1. - e*cE;
		double cv = (cE - e)/umecE;
		double sv = sq_unme2*sE/umecE;
		double anoVraie_v = Math.atan2(sv,cv);
		double unpecosv = 1. + e*cv;
		distTS = p/unpecosv;		
		double pso = petit_omega + anoVraie_v;
		double ct = Math.cos(pso); double st = Math.sin(pso);
		// Attitude Rn / Rg
		double hsg = Math.toRadians(Astro.jul2mgst_dg(ju)); // heure siderale de Greenwich
		double theta = grand_Omega - hsg;	             // longitude Noeud-Ascendant à ju
		double cln = Math.cos(theta);
		double sln = Math.sin(theta);
		double [][]m3_GH = new double [3][3];
		R3.set33(m3_GH, cln, -sln, 0, sln,  cln,  0, 0,  0,  1);
		double [][]m3_GN = new double [3][3];;
		R3.madmad(m3_GN, m3_GH, m33_HN);
		// Attitude repere orbital local Rl / Rg
		// X selon Vit, Y selon Hcin, Zl selon TS
		double [][]m3_NL = new double [3][3];
		R3.set33(m3_NL, -st, 0, ct, ct, 0, st, 0, 1, 0);
		R3.madmad(m33_GL, m3_GN, m3_NL);
		longiSat_rd = Math.atan2(m33_GL[1][2], m33_GL[0][2]);
		latiSat_rd = Math.asin(m33_GL[2][2]);
		// Position satellite dans Rg
		R3.set3(v3_TS_g, distTS*m33_GL[0][2], distTS*m33_GL[1][2], distTS*m33_GL[2][2]);  		
	}

	private void makeTopo()
	{
		double [] v3_AS_g = new double[3];
		// Segment station-Satellite dans Rg
		for (int i = 0; i < 3; i++) v3_AS_g[i] = v3_TS_g[i] - Terrien.v3_GA[i];
		// Le meme dans Ra
		R3.matvec(v3_AS_a, Terrien.m3_GA, v3_AS_g);
		double projAS = Math.sqrt(v3_AS_a[0]*v3_AS_a[0] + v3_AS_a[1]*v3_AS_a[1]);
		haut_dg = Math.toDegrees(Math.atan2(v3_AS_a[2],projAS));
		azi_dg = Math.toDegrees(Math.atan2(v3_AS_a[1], v3_AS_a[0]));
	}


	/**
	 *  Génération String donnant des informations sur la position du satellite
	 * @param ju		: date à laquelle on veut les infos
	 * @param longi_dg	: Longitude de l'observateur
	 * @param lati_dg   : Latitude de l'observateur
	 * @param alti_m    : altitude observateur en metres
	 * @param sexaDec   : si vrai affichage sexagésimal, sinon décimal
	 * @return
	 */
	public String infos(double ju, double longi_dg, double lati_dg, double alti_m, int sexaDec)
	{
//		postobsv = new Postobsv(Math.toRadians(longi_dg), Math.toRadians(lati_dg), alti_m);
		update(ju);	makeTopo();	
		double [] lLh = new double[3];
		Terrien.ccart2cgeod(lLh, v3_TS_g, 1.e-10);
		if (Math.abs(lLh[0] - longiSat_rd) > 1.e-10)
			System.out.println("Erreur appel ccart2cgeod");
		double latiSatGraf_dg = Math.toDegrees(lLh[1]);
		double altitude_km = lLh[2]/1000.;		
		DecimalFormatSymbols dfs = new DecimalFormatSymbols(); dfs.setDecimalSeparator('.');
		DecimalFormat df4 = new DecimalFormat("0.0000", dfs);
		DecimalFormat df2 = new DecimalFormat("0.00", dfs);
		String str = "";
		double longiSat_dg = Math.toDegrees(longiSat_rd);
		double latiSatCent_dg = Math.toDegrees(latiSat_rd);
		if (sexaDec > 0)
		{
			str += "Longi/Lati_centr.  : " + Utils.deg2dms(longiSat_dg)
					+ " / " + Utils.deg2dms2(latiSatCent_dg) + "\n";
			str += "Longi/Lati_geodes. : " + Utils.deg2dms2(longiSat_dg)
					+ " / " + Utils.deg2dms2(latiSatGraf_dg) + "\n";
			str += "Altitude (km) : " + df2.format(altitude_km) + "\n";			
			str += "Azimut/Hauteur : " + Utils.deg2dms(azi_dg)
					+ " / " + Utils.deg2dms2(haut_dg) + "\n";
		}
		else if (sexaDec <= 0)
		{
			str += "Longi/Lati_centr.  : " + df4.format(longiSat_dg)
					+ " / " + df4.format(latiSatCent_dg) + "\n";
			str += "Longi/Lati_geodes. : " + df4.format(longiSat_dg)	
					+ " / " + df4.format(latiSatGraf_dg)+ "\n";					
			str += "Altitude (km) : " + df2.format(altitude_km) + "\n";			
			str += "Azimut/Hauteur : " + df4.format(azi_dg)
					+ " / " + df4.format(haut_dg) + "\n";
		}

		// Calcul visibilités
		double dtmin_s = 30.; // durée minimale pour etre prise en compte
		double judeb = ju - 1./24.; // début 1 heure avant la date actuelle
		double jufin = ju + 60;      // fin 30 jours après la date actuelle
		double hDeb_h = 21.;
		double hFin_h = 23.;
		int nvismax = 10;        // Nombre max de visites retenues
		double [][] azimut = new double [3][nvismax];
		double [][] hauteur = new double [3][nvismax];
		double [][] times = new double [3][nvismax];
		int [][] iters = new int [3][nvismax];
		double hauteurMin_dg = 55. ; // hauteur minimale de visibilite

		int nvis = visibilite(hauteurMin_dg, dtmin_s, hDeb_h, hFin_h, judeb, jufin, azimut, 
				hauteur, times, iters, nvismax);
		if (nvis > 0)
		{
			if (sexaDec > 0)
			{
				for (int i = 0; i < nvis; i++)
				{
					str += "Lever(TU)  : " + Utils.jul2str1(times[0][i]) + ", Azi/Haut : " + Utils.deg2dms(azimut[0][i])
							+ " / " + Utils.deg2dms2(hauteur[0][i]) + "\n";
					str += "Hmax.(TU)  : " + Utils.jul2str1(times[1][i]) + ", Azi/Haut : " + Utils.deg2dms(azimut[1][i])
							+ " / " + Utils.deg2dms2(hauteur[1][i]) + "\n";
					str += "Coucher(TU): " + Utils.jul2str1(times[2][i]) + ", Azi/Haut : " + Utils.deg2dms(azimut[2][i])
							+ " / " + Utils.deg2dms2(hauteur[2][i]) + "\n";
				}
			}
			else if (sexaDec <= 0)
			{
				for (int i = 0; i < nvis; i++)
				{
					str += "Lever(TU)  : " + Utils.jul2str1(times[0][i]) + ", Azi/Haut : " + df4.format(azimut[0][i])
							+ " / " + df4.format(hauteur[0][i]) + "\n";
					str += "Hmax.(TU)  : " + Utils.jul2str1(times[1][i]) + ", Azi/Haut : " + df4.format(azimut[1][i])
							+ " / " + df4.format(hauteur[1][i]) + "\n";
					str += "Coucher(TU): " + Utils.jul2str1(times[2][i]) + ", Azi/Haut : " + df4.format(azimut[2][i])
							+ " / " + df4.format(hauteur[2][i]) + "\n";
				}
			}
		}
		else str += "Pas de passage satisfaisant les contraintes !";

		return str;
	}

	/**
	 *  Calcul de l'instant correspondant a une position sur orbite
	 *  dans la révolution centrée sur juIni
	 *  
	 * @param juIni  date origine du calcul
	 * @param pso    position sur orbite (angle en radian / noeud Ascendant)
	 * @return       date de passage à l'angle pso
	 */
	public double pso2time(double juIni, double pso)
	{
		double tini = (juIni - ju0)*86400; // Nb de secondes depuis ju0
		double M_ini = M0 + tini*(n0 + tini*(n0dots2 + tini*n0ddots6));
		//Calcul anomalie vraie
		double petit_omega = om0 + omP*tini;        
		double v = pso - petit_omega;				
		//Calcul anomalie excentrique
		double cos_v = Math.cos(v);
		double sin_v = Math.sin(v);
		double temp = 1 + e0*cos_v;
		double cos_E = (e0 + cos_v)/temp;
		double sin_E = sin_v* Math.sqrt(1.-e0*e0)/temp;
		double E = Math.atan2(sin_E, cos_E);
		//Calcul anomalie moyenne pour la pso
		double M = E - e0*sin_E;
		//Calcul temps écoulé entre jUini et le passage à la pso
		double n = n0 + tini*(2.*n0dots2 + tini*3.*n0ddots6);		
		double dju = (Utils.mod2(M - M_ini, 2.* Math.PI)/n)/86400.;
		// Retour instant de passage à la pso qui suit juIni dans la révolution en cours
		return juIni + dju;
	}

	// Ces programmes sont adaptés d'un ancien programme java de visualisation : CineSpace
	// dans lequel était également pris en compte un cone de visée issu du satellite
	/** n = visibilite(htmin_dg, dureeMin_s, dateDebutHorizon, dateFinHorizon, azimut[][], 
	 *		hauteur[][], times[][], iters[], nvismax);
	 * 
	 * Prevision d'un ensemble de visites possibles d'une cible par un satellite.	
	   @param htmin_dg	    Hauteur minimale
	   @param dureeMin_s  	Duree minimale acceptable pour la visibilite en secondes
	   @param hDebut		Heure décimale debut recherche
	   @param hFinal			Heure décimale fin recherche
	   @param dateDebutHorizon	Date debut recherche
	   @param dateFinHorizon	Date fin recherche
	   @param azimut			[3][nvis] 
	   @param hauteur			[3][nvis] 
	   @param times			[3][nvis] dates entree, milieu, sortie d visibilite
	   @param iters			[3][nvis] nb iterations
	   @param nvismax			Nombre max de visites a calculer
	   @return	Le nombre de visions enregistrees dans le tableau
	   Remarque : Le 1er timein est eventuellement retarde pour coincider
	   avec la datedebutHorizon, et le dernier timeout est eventuellement
	   avance pour coincider avec la datefinHorizon.
	 */	
	public int visibilite(
			double htmin_dg,
			double dureeMin_s, double hDebut, double hFinal,
			double dateDebutHorizon,
			double dateFinHorizon,
			double [][] azimut,
			double [][] hauteur,		
			double [][] times,      
			int [][] iters,
			int nvismax)
	{
		// Conversion arguments en entree
		double dureeMin_j = dureeMin_s/86400.;
		double beta = Math.toRadians(90. - htmin_dg);
		double hDeb = ((hDebut+12)/24)%1;
		double hFin = ((hFinal+12)/24)%1;
		// Tableau pour stocker les solution
		double [] sols = new double [4];
		// Calcul période orbitale médiane
		double juMed = (dateDebutHorizon+dateFinHorizon)/2.;
		double tiMed = (juMed - ju0)*86400; // Nb de secondes depuis ju0
		double Torb = 2* Math.PI/(n0 + tiMed*(2.*n0dots2 + tiMed*3.*n0ddots6));
		double jOrb = Torb/86400.;
		// Heures moyenne lever et coucher soleil dans la tranche
		double [] result = new double [7];
		Transit.getResult(new Planet(2), juMed, Math.toDegrees(Terrien.longi_rd),
				Math.toDegrees(Terrien.lati_rd), false, result, 0);
		double tlev = result[2]%1.; 
		double tpon = result[4]%1.;	
		// Variables locales de travail
		int nvis = 0; // Nombre de solutions trouvees
		double ju = dateDebutHorizon - jOrb/3.;
		// On cherche un rdv eventuel dans l'orbite [tref , tref + une periode]
		// puis une fois trouve, on passe a la periode suivante et ainsi
		// de suite jusqu'a epuiser l'horizon de recherche
		while (nvis < nvismax && ju < dateFinHorizon)
		{
			int iter;			
			iter = raffine(sols, ju, beta, 0);
			if (iter >= 0)
			{
				ju = sols[0];
				if (ju > dateFinHorizon) break;
				times[1][nvis] = ju;				
				azimut[1][nvis] = sols[1];
				hauteur[1][nvis] = sols[2];
				iters[1][nvis] = iter;
				// Si visibilité, calcul entrée et sortie
				//				if (sols[3] <= beta)
				if (sols[2] >= htmin_dg)
				{
					iter = raffine(sols, ju-jOrb/6., beta, -1);
					if (iter >= 0)
					{
						times[0][nvis] = sols[0];
						azimut[0][nvis] = sols[1];
						hauteur[0][nvis] = sols[2];
						iters[0][nvis] = iter;
						iter = raffine(sols, 2*ju-sols[0], beta, +1);
						if (iter >= 0 /* && sols[0] >= dateDebutHorizon */)
						{
							times[2][nvis] = sols[0];
							azimut[2][nvis] = sols[1];
							hauteur[2][nvis] = sols[2];
							iters[2][nvis] = iter;
							double tl = times[0][nvis]%1.;
							double tp = times[2][nvis]%1.;
							// Si durée suffisante est retenu et lever ou coucher pendant la nuit
							if (times[2][nvis] - times[0][nvis] >= dureeMin_j
									&& ((tl > tpon && tl < tlev) || (tp > tpon && tp < tlev))
									&& ((tl > hDeb && tl < hFin) || (tp > hDeb && tp < hFin))								
									) 
								nvis++;						
						}					
					}
				}
			}
			ju += jOrb;
		}
		return nvis;
	}	

	/**
	 * Calcule des angles de transit ou d'entrée ou sortie du cone de visibilité,
	 * et instant correspond dans la révolution centrée sur ju
	 * Il peut ne pas y avoir de solution (cas d'un satellite géostationnaire à l'opposé
	 * de la postobsv par exemple). La procédure itérative est donc limitée en nombre
	 * d'itérations.
	 * Trois cas de calcul :
	 * cas =  0 : Proximité maximale à la postobsv
	 * cas = -1 : Pour l'instant d'entrée dans le cone de visibilité
	 * cas = +1 : Pour l'instant de sortie du cône de visibilité
	 * 
	 * @param sols   Le tableau des valeurs calcules (t, gamma, beta)
	 * @param ju     Intant initial pour les iterations
	 * @param betaLim_rd   L'angle limite de vision beta
	 * @param cas    Cas de calcul
	 * @return
	 */
	private int raffine(double sols[], double ju, double betaLim_rd, int cas)
	{
		boolean fconv = false;
		final double cbetaLim = Math.cos(betaLim_rd);
		final double sbetaLim = Math.sin(betaLim_rd);
		final double s2betaLim = sbetaLim*sbetaLim;
		final double sinphiA_sini = Terrien.sin_phi_A * si0 ;
		double juold, hsg, theta, gA, gB, alpha0, alpha, cosSigmaLim, rS, rS2;
		double dalfa, cos_dalfa; 
		//		double cos_beta, temp, cos_sigma ; 
		int iter = 0;	// index boucle de raffinage de la solution
		do // boucle de raffinage de la solution (20 boucles ou |t-told| < 0.01)
		{
			hsg = Math.toRadians(Astro.jul2mgst_dg(ju)); // heure siderale de Greenwich
			ju2distTS(ju);	                             // etat satellite à t
			rS = distTS; 
			rS2 = rS*rS;
			theta = grand_Omega - hsg;	                 // longitude Noeud-Ascendant à ju
			// Composantes proj zA dans Rn à l'instant ju
			gA = Terrien.cos_phi_A * Math.cos(Terrien.longi_rd - theta);
			gB = Terrien.cos_phi_A * Math.sin(Terrien.longi_rd - theta)*ci0 + sinphiA_sini ;
			// Pso sat la plus proche de cette projection (pso médiane)
			alpha0 = Math.atan2(gB, gA);
			if (cas == 0) alpha = alpha0;
			else
			{
				// Modification de la pso pour entrée ou sortie du cone
				cosSigmaLim = (Terrien.distGA*s2betaLim + cbetaLim* Math.sqrt(rS2 - Terrien.distGA2*s2betaLim))/rS;
				// Calcul écart à la pso médiane
				cos_dalfa = cosSigmaLim/ Math.sqrt(gA*gA+gB*gB);
				if (cos_dalfa > 1.) cos_dalfa = 1.;
				if (cos_dalfa < -1.) cos_dalfa = -1.;
				dalfa = Math.acos(cos_dalfa);
				// Calcul pso entrée ou sortie du cone
				if (cas < 0) alpha = alpha0 - dalfa;
				else         alpha = alpha0 + dalfa;				
			}
			// Instant de passage à la pso centrée sur ju
			juold = ju;
			ju = pso2time(ju, alpha);
			// Examen si solution acceptable et si convergence 
			if (Math.abs(ju-juold) < 0.5/86400.)
			{
				// Calcul Beta
				//				cos_sigma = gA*Math.cos(alpha) + gB*Math.sin(alpha);
				//				temp = rS * cos_sigma;
				//				cos_beta = (temp - rA)/Math.sqrt(rA2 + rS2 - 2.*rA*temp);
				// Memo solutions	
				update(ju); makeTopo();
				sols[0] = ju;	
				sols[1] = azi_dg;
				sols[2] = haut_dg;
				//				sols[3] = Math.acos(cos_beta);
				//				double ttemp = (90-Math.toDegrees(sols[3]));
				//				if (Math.abs(haut_dg - ttemp) > 1.)
				//					System.out.println("Erreur hauteur : " + haut_dg + " et " + ttemp);
				fconv = true;
				break; // convergé
			}
		} while (++iter < 10);
		if (!fconv) return -1;
		return iter;
	}
}
