package fr.llibre.ml2;

public final class Terrien {
    /** Rayon equatorial Terre en m (valeur IERS 2003) */
    static double  Terre_rT = 6378136.6;
    /** Aplatissement de l'ellipsoide terrestre (valeur IERS 2003) pris nul si flagcpx < 2 */
    static double Terre_f = 1/98.25642;
    /** Parametre p de l'ellipsoide terrestre */
    static double Terre_pT = Terre_rT*(1.-Terre_f)*(1.-Terre_f);
    /** Carre de l'excentricite de l'ellipsoide terrestre */
    static  double Terre_eT2 = Terre_f*(2.-Terre_f);

    static double juOrien = 0.;

    static double longi_rd;
    static double lati_rd;
    static double rxy; // Distance station à l'axe de la Terre
    static double distGA2; // Carré distance station au centre de la Terre
    static double distGA; // Distance station au centre de la Terre
    static double cos_phi_A;
    static  double sin_phi_A;



    public static double [][]m3_CG = {{1.,0.,0.},{0.,1.,0.},{0., 0., 1.}};
    public static double [][]m3_GA = {{1.,0.,0.},{0.,1.,0.},{0., 0., 1.}};
    public static double [][]m3_CA = {{1.,0.,0.},{0.,1.,0.},{0., 0., 1.}};
    public static double []v3_GA = {1.,0.,0.};

    /** Orientation de la Terre dans le repère moyens de la date
     * On néglige la correction de nutation
     @param ju 	Instant du calcul TU en jours julien
     */
    public static void update(double ju)
    {
        final double errRotJour = 6.28318; // variation angulaire par jour
        if (Math.abs(juOrien - ju)*errRotJour >= 1.e-4)
        {
            maj_m3_CGetm3_CA(ju);
        }
        return;
    }

    private static void maj_m3_CGetm3_CA(double ju)
    {
        // heure siderale de Greenwich
        double hsg_rd = Math.toRadians(Astro.jul2mgst_dg(ju));
        double coshsg = Math.cos(hsg_rd);
        double sinhsg =  Math.sin(hsg_rd);
        m3_CG[0][0] = coshsg;
        m3_CG[1][1] = coshsg;
        m3_CG[1][0] = sinhsg;
        m3_CG[0][1] = -sinhsg;
        R3.madmad(m3_CA, m3_CG, m3_GA); // On est en repère vrai de la date
        R3.matmad(m3_CA, Astro.m3_CCvm, m3_CA); // On passe en repère moyen de la date
        juOrien = ju;
    }

    // Mise a jour repère local
    public static void setLocal_rdm(double ju, double longi, double lati, double alti)
    {
        longi_rd = longi; lati_rd = lati;
        R3.lolaAstroTom3(m3_GA, longi, lati);
        cgeod2ccart(v3_GA, longi, lati,alti);
        maj_m3_CGetm3_CA(ju);
    }

    /** Conversion coordonnees geodesiques -> cartesiennes et POLAIRES
     @param V 			Sortie V[3] : x,y,z
     @param longi 		longitude
     @param lati 		latitude geodesique
     @param alti 		altitude geodesique
     */
    public static void cgeod2ccart(double []V, double longi, double lati, double alti)
    {
        double sf = Math.sin(lati);
        double cf = Math.cos(lati);
        double unr = Math.sqrt(1. - Terre_eT2*sf*sf);
        rxy = (Terre_rT/unr + alti)*cf;
        V[0] = rxy* Math.cos(longi);
        V[1] = rxy* Math.sin(longi);
        V[2] = (Terre_pT/unr + alti)*sf;
        distGA2 = rxy*rxy + V[2]*V[2];
        distGA = Math.sqrt(distGA2);
        cos_phi_A = rxy/distGA;
        sin_phi_A = V[2]/distGA;
        return;
    }

    /** Conversion coordonnees cartesiennes -> geodesiques
     @param V 	  Sortie V[0:2] : longi, lati, h
     @param XYZ  double [3] des coordonnees cartesiennes x,y,z
     @param eps   precision relative desiree
     */
    public static int ccart2cgeod(double [] V, double [] XYZ, double eps)
    {
        double lambda = Math.atan2(XYZ[1], XYZ[0]);
        double w = XYZ[0]* Math.cos(lambda) + XYZ[1]* Math.sin(lambda);
        double h = 0., rr, rp;
        double phi = 0., sphi = 0., temp = 1.;
        double oldphi, oldh, dphi;
        double eps2 = Terre_rT * eps;
        int iter;
        for (iter = 0; iter < 10; iter++) // Converge habituellement en 2 iterations
        {
            rr = Terre_rT/temp;
            rp = Terre_pT/temp;
            oldphi = phi;
            phi = Math.atan2(XYZ[2]/(rp+h), w/(rr+h));
            sphi = Math.sin(phi);
            temp = Math.sqrt(1.-Terre_eT2*sphi*sphi);
            oldh = h;
            h = w* Math.cos(phi) + XYZ[2]*sphi -Terre_rT*temp;
            dphi = Utils.mod2(phi-oldphi,2.* Math.PI);
            if (Math.abs(dphi) < eps && Math.abs(h-oldh) < eps2) break;
        }
        V[0] = lambda;
        V[1] = phi;
        V[2] = h;
        return iter;
    }

}
