package fr.llibre.simastro;

import androidx.appcompat.app.AppCompatActivity;
//import android.support.v7.app.AppCompatActivity;
import android.content.Intent;
import android.content.res.Resources;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Arrays;
import java.util.Comparator;

/**
 * Classe Comp_obm pour trier des tableaux sur une colonne et repercuter
 * le tri sur les autres colonnes.
 */
class Comp_obm implements Comparator<CelobjParcel>
{
    int compCol;
    boolean sensInv;
    // Constructeur prenant en entrée le numéro de la colonne sur laquelle
    // s'opère le tri et un flag a true pour trier en sens inverse du naturel
    Comp_obm (int c, boolean s)
    {
        compCol = c;
        sensInv = s;
    }
    @Override
    public int compare(CelobjParcel arg0, CelobjParcel arg1)
    {
        int result;
        switch (compCol)
        {
            case 0:
                result = arg0.id.compareTo(arg1.id);
                break;
            case 1:
                result = arg0.ngc_ic -arg1.ngc_ic > 0 ? 1 : -1;
                break;
            case 2:
                result = arg0.constel.compareTo(arg1.constel);
                break;
            case 3:
                result = arg0.tipe.compareTo(arg1.tipe);
                break;
            case 4:
            {
                double r = arg0.mag - arg1.mag;
                if (r == 0.) result = 0;
                else {if (r > 0.) result = 1; else result = -1; }
            }
            break;
            case 5:
            {
                double r = arg0.dim - arg1.dim;
                if (r == 0.) result = 0;
                else {if (r > 0.) result = 1; else result = -1; }
            }
            break;
            case 6:
            {
                double r = arg0.dist - arg1.dist;
                if (r == 0.) result = 0;
                else {if (r > 0.) result = 1; else result = -1; }
            }
            break;

            default:
                result = 0;
                break;
        }
        if (sensInv) result = -result;
        return result;
    }
}


public class SelCelobjActivity extends AppCompatActivity {
    private MyArrayAdapter myAdapter;
    private ListView liste;
    private TextView tvTitres;
    private TextView tvSel;
    private CelobjParcel tabcelobj[];
    private String tabStr[];
    private String titreStr;
    private boolean modif;
    private float hChar, wChar, hChar_dp;
    private TextView tvResult;
    private int selLigne;
    private int selChamp;
    private boolean triInv[];
    private int wScreen;
    private int ncol;
    private int nbElems;
    private int [] maxLength;
    private String[] nomCols;

    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        Intent it = getIntent();
        setResult(RESULT_CANCELED, it); // Echec annoncé a priori.
        setContentView(R.layout.layout_sel_star);
        tvTitres = (TextView)findViewById(R.id.tvTitres);

        // On récupère les données
        nomCols = it.getStringArrayExtra("NOMCOLS");
        ncol = nomCols.length;
        Parcelable[] allParcelables = it.getParcelableArrayExtra("TAB_OBJETS");
        nbElems = allParcelables.length;
        tabcelobj = new CelobjParcel[nbElems];
        for (int i = 0 ; i < nbElems; i++)
        {
            tabcelobj[i] = (CelobjParcel) allParcelables[i];
            tabcelobj[i].n = i; // Pour renvoyer l'index initial de la ligne selectionnée
        }

        // FABRICATION DES LIGNES A AFFICHER
        // Initialisation des largeurs pour les différentes colonnes (en nombre de char)
        maxLength = new int[ncol]; // taille maximale de chaque colonne
        for (int i=0; i< ncol; i++)	maxLength[i] = nomCols[i].length() + 1; // + 1 pour l'espace séparateur
        // Détermination des largeurs maximales pour les différentes colonnes
        DecimalFormatSymbols sb = new DecimalFormatSymbols(); sb.setDecimalSeparator('.');
        DecimalFormat df1 = new DecimalFormat("0.0", sb);
        int lmax;
        // Largeur des colonnes strings
        String str;
        for(int i = 0 ; i < nbElems ; i++)
        {
            str = Integer.toString(tabcelobj[i].ngc_ic);
            lmax = tabcelobj[i].id.length()+1 ; if (lmax > maxLength[0]) maxLength[0] = lmax;
            lmax = str.length() + 1 ; if (lmax > maxLength[1]) maxLength[1] = lmax;
            lmax = tabcelobj[i].constel.length()+1 ; if (lmax > maxLength[2]) maxLength[2] = lmax;
            lmax = tabcelobj[i].tipe.length()+1 ; if (lmax > maxLength[3]) maxLength[3] = lmax;
            lmax = (df1.format(tabcelobj[i].mag)).length()+1 ; if (lmax > maxLength[4]) maxLength[4] = lmax;
            lmax = (df1.format(tabcelobj[i].dim)).length()+1 ; if (lmax > maxLength[5]) maxLength[5] = lmax;
            lmax = (df1.format(tabcelobj[i].dist)).length()+3 ; if (lmax > maxLength[6]) maxLength[6] = lmax;
            lmax = tabcelobj[i].nom.length()+1 ; if (lmax > maxLength[7]) maxLength[7] = lmax;
        }
        // Largeur totale des colonnes en nombre de caractères
        int tot = 0;
        for (int i=0; i< ncol; i++) tot += maxLength[i];
        // On génère le String de la ligne de TITRE, avec répartition des espaces
        int m, l, p, q;
        titreStr = new String("");
        // String en tête, espaces en queue
        titreStr = nomCols[0]; for(int k = nomCols[0].length(); k < maxLength[0]; k++) titreStr += " ";
        // String au milieu, espaces avant et après
        m = maxLength[1]; l = nomCols[1].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " "; titreStr += nomCols[1]; for(int k = 0; k < q; k++) titreStr += " ";
        // String au milieu, espaces avant et après
        m = maxLength[2]; l = nomCols[2].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " "; titreStr += nomCols[2]; for(int k = 0; k < q; k++) titreStr += " ";
        // String au milieu, espaces avant et après
        m = maxLength[3]; l = nomCols[3].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " "; titreStr += nomCols[3]; for(int k = 0; k < q; k++) titreStr += " ";
        // String au milieu, espaces avant et après
        m = maxLength[4]; l = nomCols[4].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " "; titreStr += nomCols[4]; for(int k = 0; k < q; k++) titreStr += " ";
        // String au milieu, espaces avant et après
        m = maxLength[5]; l = nomCols[5].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " "; titreStr += nomCols[5]; for(int k = 0; k < q; k++) titreStr += " ";

        // String au milieu, espaces avant et après
        m = maxLength[6]; l = nomCols[6].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " "; titreStr += nomCols[6]; for(int k = 0; k < q; k++) titreStr += " ";
        // String au milieu, espaces avant
        m = maxLength[7]; l = nomCols[7].length(); q = (m - l)/2; p = m -l - q;
        for(int k = 0; k < p; k++) titreStr += " ";  titreStr += nomCols[7];

        // On génère les String des LIGNES a afficher, avec meme répartition des espaces
        tabStr = new String[nbElems];
        for (int i = 0; i < nbElems; i++)
        {
            // String en tête, espaces en queue
            str = tabcelobj[i].id;
            tabStr[i]  = str; for(int k = str.length(); k < maxLength[0]; k++) tabStr[i] += " ";

            // String au milieu, espaces avant et après
            str = Integer.toString(tabcelobj[i].ngc_ic);
            m = maxLength[1]; l = str.length(); q = (m - l)/2; p = m -l - q;
            for(int k = 0; k < p; k++) tabStr[i] += " "; tabStr[i] += str; for(int k = 0; k < q; k++) tabStr[i] += " ";

            // String au milieu, espaces avant et après
            m = maxLength[2]; l = tabcelobj[i].constel.length(); q = (m - l)/2; p = m -l - q;
            for(int k = 0; k < p; k++) tabStr[i] += " "; tabStr[i] += tabcelobj[i].constel; for(int k = 0; k < q; k++) tabStr[i] += " ";

            // String au milieu, espaces avant et après
            m = maxLength[3]; l = tabcelobj[i].tipe.length(); q = (m - l)/2; p = m -l - q;
            for(int k = 0; k < p; k++) tabStr[i] += " "; tabStr[i] += tabcelobj[i].tipe; for(int k = 0; k < q; k++) tabStr[i] += " ";

            str = df1.format(tabcelobj[i].mag);
            // String au milieu, espaces avant et après
            m = maxLength[4]; l = str.length(); q = (m - l)/2; p = m -l - q;
            for(int k = 0; k < p; k++) tabStr[i] += " "; tabStr[i] += str; for(int k = 0; k < q; k++) tabStr[i] += " ";

            str = df1.format(tabcelobj[i].dim);
            // String au milieu, espaces avant et après
            m = maxLength[5]; l = str.length(); q = (m - l)/2; p = m -l - q;
            for(int k = 0; k < p; k++) tabStr[i] += " "; tabStr[i] += str; for(int k = 0; k < q; k++) tabStr[i] += " ";

            str = df1.format(tabcelobj[i].dist);
            // String au milieu, espaces avant et après
            m = maxLength[6]; l = str.length(); q = (m - l)/2; p = m -l - q;
            for(int k = 0; k < p; k++) tabStr[i] += " "; tabStr[i] += str; for(int k = 0; k < q; k++) tabStr[i] += " ";

            str = tabcelobj[i].nom;
            // String en tête
            tabStr[i] += str;

            tabcelobj[i].all = new String(tabStr[i]);
        }

        // Calcul taille des lettres pour tout faire contenir sur une ligne
        // Pour accéder à la taille des TextView de la page sélection
        // Densité et taille écran
        Resources res = this.getResources();
        DisplayMetrics dm = res.getDisplayMetrics();
        float density = dm.density;
        wScreen = dm.widthPixels;
        // Nombre de lettres affichables par ligne
        hChar = tvTitres.getTextSize();
        wChar = hChar*0.6f;
        int nbdispo = (int) (wScreen/wChar);
        // Calcul largeur des lettres et hauteur à spécifier éventuellement
        // si on est obligé de la diminuer
        modif = false; // Pas de modif à faire a priori
        if (tot > nbdispo) // Diminution de la taille des lettres
        {
            nbdispo = tot;
            wChar = ((float) wScreen)/nbdispo;
            hChar = wChar/0.6f;
            modif = true;
        }
        hChar_dp = hChar/density; //A spécifier par setTextSize(v)

        // On initialise le sens des tris des colonnes (est inverse avant chaque tri)
        triInv = new boolean[ncol];
        for (int i = 0 ; i < ncol; i++) triInv[i] = true;

        // Creation ArrayAdapter d'affichage dans le ListView
        myAdapter = new MyArrayAdapter();

        tvTitres = (TextView)findViewById(R.id.tvTitres);
        tvTitres.setText(titreStr);
        tvTitres.setOnTouchListener(MyOnTouchListener);
        if (modif) tvTitres.setTextSize(hChar_dp);
        tvResult = (TextView)findViewById(R.id.tvResult);
        liste = (ListView) findViewById(R.id.list);;
        liste.setDrawSelectorOnTop(false);
        // Conditionnement du listView et de son ArrayAdapter
        liste.setOnItemClickListener(cl);
        startSelect();
    }

    // (Re)Demarrage du travail d'affichage-sélection
    public void startSelect()
    {
        liste.setAdapter(myAdapter);
        tvResult.setText("Sélectionnez une ligne ou colonne");
        selLigne = -1; // Rien n'est sélectionné
        selChamp = -1;
    }

    // Callback pour la sélection d'une ligne
    private OnItemClickListener cl = new OnItemClickListener(){
        @Override
        public void onItemClick(AdapterView<?> parent, View v, int position, long id)
        {
            if (selLigne >= 0) tvSel.setBackgroundColor(Color.WHITE);
            selLigne = position;
            TextView label = (TextView) v.findViewById(R.id.tvLigne);
            label.setBackgroundColor(Color.YELLOW);
            tvSel = label;
            tvResult.setText("OK pour sélectionner " + tabcelobj[selLigne].id);
        }
    };

    // Callbacks boutons Ok et CANCEL provoquant la fin de l'activité avec renvoi du résultat
    public void onOkCancel(View w)
    {
        Intent it = new Intent();
        switch(w.getId())
        {
            case R.id.button_oui :
                if (selLigne >= 0)
                {
                    it.putExtra("score", tabcelobj[selLigne].n);
                    setResult(RESULT_OK, it);
                }
                else setResult(RESULT_CANCELED, it);
                break;
            case R.id.button_non :
                setResult(RESULT_CANCELED, it);
                break;
        }
        finish();

    }

    class MyArrayAdapter extends ArrayAdapter<String>
    {
        MyArrayAdapter () // Le contructeur appelle le super_constructeur
        {
            super(SelCelobjActivity.this, R.layout.ligne, R.id.tvLigne, tabStr);
        }
        // On surcharge uniquement cette méthode
        public View getView(int pos, View vue, ViewGroup parent)
        {
            if (vue == null) vue = super.getView(pos, vue, parent);
            TextView label = (TextView) vue.findViewById(R.id.tvLigne);
            if (modif) label.setTextSize(hChar_dp);
            if (pos == selLigne) {
                label.setBackgroundColor(Color.YELLOW);
                tvSel = label;
            }
            else label.setBackgroundColor(Color.WHITE);;

            label.setText(tabStr[pos]);
            return(vue);
        }
    }

    // Routine de traitement des actions des doigts sur la ligne de Titres
    // On ne traite que le levé (pseudo-click), mais qui permet de récuperer
    // la position du doigt pour connaitre le champ cliqué
    OnTouchListener MyOnTouchListener = new OnTouchListener(){
        @Override
        public boolean onTouch(View view, MotionEvent event) {

            switch(event.getAction() & MotionEvent.ACTION_MASK)
            {
                case MotionEvent.ACTION_DOWN:
                case MotionEvent.ACTION_POINTER_DOWN:
                case MotionEvent.ACTION_MOVE:
                case MotionEvent.ACTION_POINTER_UP:
                    break;
                case MotionEvent.ACTION_UP:
                    // On supprime la sélection de ligne
                    selLigne = -1;
                    // détermination de la colonne sélectionnée
                    selChamp = -1;
                    int nn = 0,  nc = (int) (event.getX()/wChar);
                    for (int i = 0; i < ncol; i++)
                    {
                        nn += maxLength[i];
                        if (nc < nn) {selChamp = i; break;}
                    }
                    if (selChamp < 0) selChamp = ncol-1;
                    // On inverse l'ordre de tri de la colonne sélectionnée
                    boolean nouveau = !triInv[selChamp];
                    for (int i = 0; i < ncol; i++) triInv[i] = true; // Raz de tous les sens
                    triInv[selChamp] = nouveau;  // sauf le selectionné qui est inversé
                    // On trie le tableau Messier
                    Arrays.sort(tabcelobj, 0, nbElems, new Comp_obm(selChamp, nouveau));
                    // On crée un nouveau tableau à afficher
                    for (int i = 0; i < nbElems; i++) tabStr[i] = tabcelobj[i].all;
                    // On rappelle le sélecteur avec ce nouveau tableau
                    startSelect();
                    break;
            }
            return true;
        }
    };
}
