package fr.llibre.simastro;
import androidx.appcompat.app.AppCompatActivity;
//import android.support.v7.app.AppCompatActivity;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.util.Log;
import android.widget.ProgressBar;

import static java.lang.Boolean.FALSE;
import static java.lang.Boolean.TRUE;

/**
 * Created by llibre on 30/09/2017.
 */

public class ViseeAcq extends AppCompatActivity implements SensorEventListener {
    ProgressBar pb;
    static double dureeMax = 5., tau = 0.3;
    static long timeIni, timeOld;
    long time;
    static boolean debutGrav, debutMag;
    SensorManager sensorMgr;
    boolean		MagnetSupported;
    boolean		gravitySupported;
    static float[] gravMes = new float[3];
    static float[] magnMes = new float[3];
    static double[] gravFil = new double[3];
    static double[] magnFil = new double[3];
    static double [] petitY = new double[3];

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.visee_progress);
        pb = (ProgressBar) findViewById(R.id.progressBar);
        pb.setMax(100);
        debutGrav = FALSE;
        debutMag = FALSE;
    }

    protected void onStart()
    {
        super.onStart();
        if (!enregistrement()) {
            setResult(RESULT_CANCELED);
            finish();
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        desenregistrement();
    }


    public void onAccuracyChanged(Sensor sensor, int accuracy) {
        switch(accuracy){
            case SensorManager.SENSOR_STATUS_ACCURACY_HIGH:
            case SensorManager.SENSOR_STATUS_ACCURACY_MEDIUM:
            case SensorManager.SENSOR_STATUS_ACCURACY_LOW:
            case SensorManager.SENSOR_STATUS_UNRELIABLE:
        }
        Log.d("Sensor",sensor.getType()+":"+accuracy);
    }

    boolean enregistrement() {
        boolean sensorOk = TRUE;
        sensorMgr = (SensorManager) getSystemService(SENSOR_SERVICE);

        gravitySupported = sensorMgr.registerListener(this,sensorMgr.getDefaultSensor(Sensor.TYPE_GRAVITY), SensorManager.SENSOR_DELAY_UI);
        if (!gravitySupported){
            sensorMgr.unregisterListener(this,sensorMgr.getDefaultSensor(Sensor.TYPE_GRAVITY));
            sensorOk = FALSE;
        }

        MagnetSupported = sensorMgr.registerListener(this,sensorMgr.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD), SensorManager.SENSOR_DELAY_UI);
        if (!MagnetSupported) {
            sensorMgr.unregisterListener(this,sensorMgr.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD));
            sensorOk = FALSE;
        }
        return sensorOk;
    }

    void desenregistrement() {
        if (gravitySupported) sensorMgr.unregisterListener(this,sensorMgr.getDefaultSensor(Sensor.TYPE_GRAVITY));
        if (MagnetSupported)  sensorMgr.unregisterListener(this,sensorMgr.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD));
    }

    public void onSensorChanged(SensorEvent event) {
        time = event.timestamp;
        switch(event.sensor.getType())
        {
            case Sensor.TYPE_GRAVITY:
                onGravityChanged(event);
                break;
            case Sensor.TYPE_MAGNETIC_FIELD:
                onMagneticFieldChanged(event);
                break;
        }
    }

    private void onGravityChanged(SensorEvent event)
    {
        float x, y, z;
        gravMes[0] = x = event.values[0];
        gravMes[1] = y = event.values[1];
        gravMes[2] = z = event.values[2];
        if (!debutGrav && debutMag)
        {
            timeIni= timeOld = time;
        }
        debutGrav = TRUE;
        if (debutMag) orientationChanged();
    }

    private void onMagneticFieldChanged(SensorEvent event){
        float x,y,z;
        magnMes[0] = x = event.values[0];
        magnMes[1] = y = event.values[1];
        magnMes[2] = z = event.values[2];
        if (!debutMag && debutGrav) {
            timeIni= timeOld = time;
        }
        debutMag = TRUE;
        if (debutGrav) orientationChanged();
    }

    private void orientationChanged()
    {
        double alpha = 0.1, unmalf = 1. - alpha;
        double duree =(time - timeIni)*1.e-9;
        pb.setProgress((int) (100.*duree/ dureeMax));
        if (duree == 0)
        {
            for(int n = 0; n < 3; n++) { gravFil[n] = gravMes[n]; magnFil[n] = magnMes[n];}
        }
        else for(int n = 0; n < 3; n++)
        {
            gravFil[n] = alpha*gravMes[n] + unmalf*gravFil[n];
            magnFil[n] = alpha*magnMes[n] + unmalf*magnFil[n];
        }

        double [] J = new double[3]; // Axe horizontal dirigé vers le Nord
        double [] K = new double[3]; // Axe vertical dirigé vers le zénit
        double Iy; // Comp syr y de l'axe horizontal dirigé vers l'Est
        double gmod =  Math.sqrt(gravMes[0]* gravMes[0] + gravMes[1]* gravMes[1] + gravMes[2]* gravMes[2]);
        for(int n = 0; n < 3; n++) K[n] = gravMes[n]/gmod;
        double mk = magnMes[0]*K[0] + magnMes[1]*K[1] + magnMes[2]*K[2];
        for(int n = 0; n < 3; n++) J[n] = magnMes[n] - mk*K[n];
        double jmod = Math.sqrt(J[0]*J[0] + J[1]*J[1] +J[2]*J[2]);
        for(int n = 0; n < 3; n++) J[n] /= jmod;
//        I[0] = J[1]*K[2] - J[2]*K[1];
        Iy = J[2]*K[0] - J[0]*K[2];
        //       I[2] = J[0]*K[1] - J[1]*K[0];

        double [] petitYmes  ={Iy, J[1], K[1]};
        if (duree == 0)
        {
            petitY[0] = petitYmes[0]; petitY[1] = petitYmes[1]; petitY[2] = petitYmes[2];
        }
        else {
            double deltaT = 1.e-9 * (time - timeOld);
            timeOld = time;
            double sum = tau + deltaT;
            for (int i = 0; i < 3; i++) {
                petitY[i] = (tau * petitY[i] + deltaT * petitYmes[i]) / sum;
            }
            if (duree >= dureeMax) {
                double cap = Math.toDegrees(Math.atan2(-petitY[0], petitY[1]));
                double ass = Math.toDegrees(Math.asin(petitY[2]));
                Intent it = new Intent();
                it.putExtra("cap", cap);
                it.putExtra("ass", ass);
                setResult(RESULT_OK, it);
                finish();
            }
        }
    }
}
